###############################################################################
# Name: reads_quality_control.smk
# Author: Nicolas Fernandez
# Affiliation: IRD_U233_TransVIHMI
# Aim: Snakefile for Reads Quality Control pipeline
# Date: 2021.04.30
# Run: snakemake --snakefile reads_quality_control.smk --cores --use-conda 
# Latest modification: 2022.03.31
# Todo: done

###############################################################################
# PUBLICATIONS #


###############################################################################
# CONFIGURATION #
configfile: "config/config.yaml"

###############################################################################
# FUNCTIONS #

###############################################################################
# WILDCARDS #
SAMPLE, = glob_wildcards("resources/reads/{sample}.fastq.gz")

###############################################################################
# ENVIRONMENTS #
FASTQC = config["conda"]["fastqc"]            # FastQC
FASTQSCREEN = config["conda"]["fastq-screen"] # FastQ Screen
MULTIQC = config["conda"]["multiqc"]          # MultiQC

###############################################################################
# RESOURCES #
CPUS = config["resources"]["cpus"]     # Resources thread

###############################################################################
# PARAMETERS #
CONFIG = config["fastq-screen"]["config"]   # Fastq-screen --conf
ALIGNER = config["fastq-screen"]["aligner"] # Fastq-screen --aligner
SUBSET = config["fastq-screen"]["subset"]   # Fastq-screen --subset

###############################################################################
rule all:
    input:
        multiqc = "results/multiqc/"

###############################################################################
rule multiqc_reports_aggregation:
    # Aim: aggregates bioinformatics analyses results into a single report
    # Use: multiqc [OPTIONS] --output [MULTIQC/] [FASTQC/] [MULTIQC/] 
    message:
        "MultiQC reports aggregation"
    conda:
        MULTIQC
    input:
        fastqc = expand("results/fastqc/{sample}",
                        sample = SAMPLE),
        fastqscreen = expand ("results/fastq-screen/{sample}",
                              sample = SAMPLE)
    output:
        multiqc = directory("results/multiqc/")
    log:
        "results/reports/multiqc.log"
    shell:
        "multiqc "                  # Multiqc, searches in given directories for analysis & compiles a HTML report
        "--quiet "                   # -q: Only show log warning
        "--outdir {output.multiqc} " # -o: Create report in the specified output directory
        "{input.fastqc} "            # Input FastQC files
        "{input.fastqscreen} "       # Input Fastq-Screen
        "--no-ansi "                 # Disable coloured log
        "&> {log}"                   # Add redirection for log

###############################################################################
rule fastqscreen_contamination_checking:
    # Aim: screen if the composition of the library matches with  what you expect
    # Use: fastq_screen [OPTIONS] --outdir [DIR/] [SAMPLE_1.fastq] ... [SAMPLE_n.fastq]
    priority: 42
    message:
        "Fastq-Screen reads contamination check for {wildcards.sample} sample"
    conda:
        FASTQSCREEN
    resources:
        cpus = CPUS
    params:
        config = CONFIG,
        aligner = ALIGNER,
        subset = SUBSET
    input:
        fastq = "resources/reads/{sample}.fastq.gz"
    output:
        fastqscreen = directory("results/fastq-screen/{sample}/")
    log:
        "results/reports/fastq-screen/{sample}.log"
    shell:
        "mkdir -p {output.fastqscreen} " # (*) this directory must exist as the program will not create it
        "2> /dev/null && "               # in silence and then... 
        "fastq_screen "                  # FastqScreen, what did you expect ?
        "-q "                             # --quiet: Only show log warning
        "--threads {resources.cpus} "     # --threads: Specifies across how many threads bowtie will be allowed to run
        "--conf {params.config} "         # path to configuration file
        "--aligner {params.aligner} "     # -a: choose aligner 'bowtie', 'bowtie2', 'bwa'
        "--subset {params.subset} "       # Don't use the whole sequence file, but create a subset of specified size
        "--outdir {output.fastqscreen} "  # Output directory
        "{input.fastq} "                  # Input file.fastq
        "&> {log}"                        # Add redirection for log

###############################################################################
rule fastqc_quality_control:
    # Aim: reads sequence files and produces a quality control report
    # Use: fastqc [OPTIONS] --output [DIR/] [SAMPLE_1.fastq] ... [SAMPLE_n.fastq]
    priority: 42
    message:
        "FastQC reads quality control for {wildcards.sample} sample"
    conda:
        FASTQC
    resources:
        cpus = CPUS
    input:
        fastq = "resources/reads/{sample}.fastq.gz"
    output:
        fastqc = directory("results/fastqc/{sample}/")
    log:
        "results/reports/fastqc/{sample}.log"
    shell:
        "mkdir -p {output.fastqc} " # (*) this directory must exist as the program will not create it
        "2> /dev/null && "          # in silence and then... 
        "fastqc "                    # FastQC, a high throughput sequence QC analysis tool
        "--quiet "                    # -q: Supress all progress messages on stdout and only report errors
        "--threads {resources.cpus} " # -t: Specifies files number which can be processed simultaneously
        "--outdir {output.fastqc} "   # -o: Create all output files in the specified output directory
        "{input.fastq} "              # Input file.fastq
        "&> {log}"                    # Add redirection for log

###############################################################################
